@@ -1,22 +1,29 @@
-# $Id: LibXML.pm,v 1.48 2002/05/08 00:27:03 phish Exp $
+# $Id: LibXML.pm,v 1.59 2002/05/20 10:36:49 phish Exp $
 
 package XML::LibXML;
 
 use strict;
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS
+            $skipDTD $skipXMLDeclaration $setTagCompression
+            $MatchCB $ReadCB $OpenCB $CloseCB );
 use Carp;
 use XML::LibXML::NodeList;
 use IO::Handle; # for FH reads called as methods
 
-$VERSION = "1.49";
+$VERSION = "1.50";
 require Exporter;
 require DynaLoader;
 
 @ISA = qw(DynaLoader Exporter);
 
-$XML::LibXML::skipDTD            = 0;
-$XML::LibXML::skipXMLDeclaration = 0;
-$XML::LibXML::setTagCompression  = 0;
+$skipDTD            = 0;
+$skipXMLDeclaration = 0;
+$setTagCompression  = 0;
+
+$MatchCB = undef;
+$ReadCB  = undef;
+$OpenCB  = undef;
+$CloseCB = undef;
 
 bootstrap XML::LibXML $VERSION;
 
@@ -102,42 +109,83 @@ bootstrap XML::LibXML $VERSION;
 sub new {
     my $class = shift;
     my %options = @_;
+    if ( not exists $options{XML_LIBXML_KEEP_BLANKS} ) {
+        $options{XML_LIBXML_KEEP_BLANKS} = 1;
+    }
+
     my $self = bless \%options, $class;
+    if ( defined $options{Handler} ) {
+        $self->set_handler( $options{Handler} );
+    }
     return $self;
 }
 
 sub match_callback {
     my $self = shift;
-    $self->{XML_LIBXML_MATCH_CB} = shift if scalar @_;
-    return $self->{XML_LIBXML_MATCH_CB};
+    if ( ref $self ) {
+        $self->{XML_LIBXML_MATCH_CB} = shift if scalar @_;
+        return $self->{XML_LIBXML_MATCH_CB};
+    }
+    else {
+        $MatchCB = shift if scalar @_;
+        return $MatchCB;
+    }
 }
 
 sub read_callback {
     my $self = shift;
-    $self->{XML_LIBXML_READ_CB} = shift if scalar @_;
-    return $self->{XML_LIBXML_READ_CB};
+    if ( ref $self ) {
+        $self->{XML_LIBXML_READ_CB} = shift if scalar @_;
+        return $self->{XML_LIBXML_READ_CB};
+    }
+    else {
+        $ReadCB = shift if scalar @_;
+        return $ReadCB;
+    }
 }
 
 sub close_callback {
     my $self = shift;
-    $self->{XML_LIBXML_CLOSE_CB} = shift if scalar @_;
-    return $self->{XML_LIBXML_CLOSE_CB};
+    if ( ref $self ) {
+        $self->{XML_LIBXML_CLOSE_CB} = shift if scalar @_;
+        return $self->{XML_LIBXML_CLOSE_CB};
+    }
+    else {
+        $CloseCB = shift if scalar @_;
+        return $CloseCB;
+    }
 }
 
 sub open_callback {
     my $self = shift;
-    $self->{XML_LIBXML_OPEN_CB} = shift if scalar @_;
-    return $self->{XML_LIBXML_OPEN_CB};
+    if ( ref $self ) {
+        $self->{XML_LIBXML_OPEN_CB} = shift if scalar @_;
+        return $self->{XML_LIBXML_OPEN_CB};
+    }
+    else {
+        $OpenCB = shift if scalar @_;
+        return $OpenCB;
+    }
 }
 
 sub callbacks {
     my $self = shift;
-    if (@_) {
-        my ($match, $open, $read, $close) = @_;
-        @{$self}{qw(XML_LIBXML_MATCH_CB XML_LIBXML_OPEN_CB XML_LIBXML_READ_CB XML_LIBXML_CLOSE_CB)} = ($match, $open, $read, $close);
+    if ( ref $self ) {
+        if (@_) {
+            my ($match, $open, $read, $close) = @_;
+            @{$self}{qw(XML_LIBXML_MATCH_CB XML_LIBXML_OPEN_CB XML_LIBXML_READ_CB XML_LIBXML_CLOSE_CB)} = ($match, $open, $read, $close);
+        }
+        else {
+            return @{$self}{qw(XML_LIBXML_MATCH_CB XML_LIBXML_OPEN_CB XML_LIBXML_READ_CB XML_LIBXML_CLOSE_CB)};
+        }
     }
     else {
-        return @{$self}{qw(XML_LIBXML_MATCH_CB XML_LIBXML_OPEN_CB XML_LIBXML_READ_CB XML_LIBXML_CLOSE_CB)};
+        if (@_) {
+           ( $MatchCB, $OpenCB, $ReadCB, $CloseCB ) = @_;
+        }
+        else {
+            return ( $MatchCB, $OpenCB, $ReadCB, $CloseCB );
+        }
     }
 }
 
@@ -190,6 +238,40 @@ sub base_uri {
     return $self->{XML_LIBXML_BASE_URI};
 }
 
+sub set_handler {
+    my $self = shift;
+    if ( defined $_[0] ) {
+        $self->{HANDLER} = $_[0];
+
+        $self->{SAX} = {State => 0,
+                        ELSTACK    => []};
+    }
+    else {
+        # undef SAX handling
+        delete $self->{HANDLER};
+        delete $self->{SAX};
+    }
+}
+
+sub _auto_expand {
+    my ( $self, $result, $uri ) = @_;
+
+    $result->setBaseURI( $uri ) if defined $uri;
+
+    if ( defined $self->{XML_LIBXML_EXPAND_XINCLUDE}
+         and  $self->{XML_LIBXML_EXPAND_XINCLUDE} == 1 ) {
+        $self->{_State_} = 1;
+        eval { $self->processXIncludes($result); };
+            my $err = $@;
+        $self->{_State_} = 0;
+        if ($err) {
+            $result = undef;
+            croak $err;
+        }
+    }
+    return $result;
+}
+
 sub parse_string {
     my $self = shift;
     croak("parse already in progress") if $self->{_State_};
@@ -201,30 +283,26 @@ sub parse_string {
     $self->{_State_} = 1;
     my $result;
 
-    eval {
-        $result = $self->_parse_string( @_ );
-    };
-
-    my $err = $@;
-    $self->{_State_} = 0;
-    if ($err) {
-        croak $err;
+    if ( defined $self->{SAX} ) {
+        my $string = shift;
+        eval { $self->_parse_sax_string($string); };
+        my $err = $@;
+        $self->{_State_} = 0;
+        if ($err) {
+            croak $err;
+        }
     }
+    else {
+        eval { $result = $self->_parse_string( @_ ); };
 
-    my $uri = $self->{XML_LIBXML_BASE_URI};
-    $result->setBaseURI( $uri ) if defined $uri;
+        my $err = $@;
+        $self->{_State_} = 0;
+        if ($err) {
+            croak $err;
+        }
 
-    if ( defined $self->{XML_LIBXML_EXPAND_XINCLUDE}
-         and  $self->{XML_LIBXML_EXPAND_XINCLUDE} == 1 ) {
-         $self->{_State_} = 1;
-         eval { $self->processXIncludes($result); };
-         my $err = $@;
-         $self->{_State_} = 0;
-         if ($err) {
-             $result = undef;
-             croak $err;
-         }
-     }
+        $result = $self->_auto_expand( $result, $self->{XML_LIBXML_BASE_URI} );
+    }
 
     return $result;
 }
@@ -234,29 +312,24 @@ sub parse_fh {
     croak("parse already in progress") if $self->{_State_};
     $self->{_State_} = 1;
     my $result;
-    eval {
-        $result = $self->_parse_fh( @_ );
-    };
-    my $err = $@;
-    $self->{_State_} = 0;
-    if ($err) {
-        croak $err;
-    }
-
-    my $uri = $self->{XML_LIBXML_BASE_URI} ;
-    $result->setBaseURI( $uri ) if defined $uri;
-
-    if ( defined $self->{XML_LIBXML_EXPAND_XINCLUDE}
-         and  $self->{XML_LIBXML_EXPAND_XINCLUDE} == 1 ) {
-         $self->{_State_} = 1;
-         eval { $self->processXIncludes($result); };
-         my $err = $@;
-         $self->{_State_} = 0;
-         if ($err) {
-            $result = undef;
+    if ( defined $self->{SAX} ) {
+        eval { $self->_parse_sax_fh( @_ );  };
+        my $err = $@;
+        $self->{_State_} = 0;
+        if ($err) {
             croak $err;
-         }
-     }
+        }
+    }
+    else {
+        eval { $result = $self->_parse_fh( @_ ); };
+        my $err = $@;
+        $self->{_State_} = 0;
+        if ($err) {
+            croak $err;
+        }
+
+        $result = $self->_auto_expand( $result,, $self->{XML_LIBXML_BASE_URI} );
+    }
 
     return $result;
 }
@@ -266,30 +339,24 @@ sub parse_file {
     croak("parse already in progress") if $self->{_State_};
     $self->{_State_} = 1;
     my $result;
-    eval {
-        $result = $self->_parse_file(@_);
-    };
-    my $err = $@;
-    $self->{_State_} = 0;
-    if ($err) {
-        croak $err;
+    if ( defined $self->{SAX} ) {
+        eval { $self->_parse_sax_file( @_ );  };
+        my $err = $@;
+        $self->{_State_} = 0;
+        if ($err) {
+            croak $err;
+        }
     }
+    else {
+        eval { $result = $self->_parse_file(@_); };
+        my $err = $@;
+        $self->{_State_} = 0;
+        if ($err) {
+            croak $err;
+        }
 
-    # files will not get a base dir, since they are based by their
-    # filename.
-
-    if ( defined $self->{XML_LIBXML_EXPAND_XINCLUDE}
-         and  $self->{XML_LIBXML_EXPAND_XINCLUDE} == 1 ) {
-
-         $self->{_State_} = 1;
-         eval { $self->processXIncludes($result); };
-         my $err = $@;
-         $self->{_State_} = 0;
-         if ($err) {
-             $result = undef;
-             croak $err;
-         }
-     }
+        $result = $self->_auto_expand( $result );
+    }
 
     return $result;
 }
@@ -306,10 +373,13 @@ sub parse_xml_chunk {
     }
 
     $self->{_State_} = 1;
+    if ( defined $self->{SAX} ) {
+        eval { $result = $self->_parse_sax_xml_chunk( @_ ); };
+    }
+    else {
+        eval { $result = $self->_parse_xml_chunk( @_ ); };
+    }
 
-    eval {
-        $result = $self->_parse_xml_chunk( @_ );
-    };
     my $err = $@;
     $self->{_State_} = 0;
     if ($err) {
@@ -325,6 +395,59 @@ sub processXIncludes {
     return $self->_processXIncludes($doc || " ");
 }
 
+sub init_push {
+    my $self = shift;
+
+    if ( defined $self->{CONTEXT} ) {
+        delete $self->{COMTEXT};
+    }
+
+    if ( defined $self->{SAX} ) {
+        $self->{CONTEXT} = $self->_start_push(1);
+    }
+    else {
+        $self->{CONTEXT} = $self->_start_push(0);
+    }
+}
+
+
+sub push {
+    my $self = shift;
+
+    if ( not defined $self->{CONTEXT} ) {
+        if ( defined $self->{SAX} ) {
+            $self->{CONTEXT} = $self->_start_push(1);
+        }
+        else {
+            $self->{CONTEXT} = $self->_start_push(0);
+        }
+    }
+
+    foreach ( @_ ) {
+        $self->_push( $self->{CONTEXT}, $_ );
+    }
+}
+
+sub finish_push {
+    my $self = shift;
+    my $restore = shift || 0;
+    return undef unless defined $self->{CONTEXT};
+
+    my $retval;
+
+    if ( defined $self->{SAX} ) {
+        eval { $retval = $self->_end_sax_push( $self->{CONTEXT} ); };
+    }
+    else {
+        eval { $retval = $self->_end_push( $self->{CONTEXT}, $restore ); };
+    }
+    delete $self->{CONTEXT};
+    if ( $@ ) {
+        croak( $@ );
+    }
+    return $retval;
+}
+
 sub __read {
     read($_[0], $_[1], $_[2]);
 }
@@ -349,6 +472,8 @@ sub isSupported {
     return $self->can($feature) ? 1 : 0;
 }
 
+sub getChildNodes { my $self = shift; return $self->childNodes(); }
+
 sub childNodes {
     my $self = shift;
     my @children = $self->_childNodes();
@@ -398,6 +523,11 @@ sub find {
     return undef;
 }
 
+sub setOwnerDocument {
+    my ( $self, $doc ) = @_;
+    $doc->adoptNode( $self );
+}
+
 1;
 
 package XML::LibXML::Document;
@@ -438,6 +568,11 @@ sub toString {
     return $retval;
 }
 
+sub process_xinclude {
+    my $self = shift;
+    XML::LibXML->new->processXIncludes( $self );
+}
+
 1;
 
 package XML::LibXML::DocumentFragment;
@@ -763,6 +898,74 @@ sub removeNamedItemNS {
 
 1;
 
+package XML::LibXML::_SAXParser;
+
+# this is pseudo class!!!
+use Carp;
+
+sub start_document {
+    my $parser = shift;
+    $parser->{SAX}->{State} = 1;
+    $parser->{HANDLER}->start_document({});
+}
+
+sub end_document {
+    my $parser = shift;
+    $parser->{SAX}->{State} = 0;
+}
+
+sub xml_decl {
+    my ( $parser, $version, $encoding ) = @_;
+
+    my $decl = {version => $version};
+    $decl->{encoding} = $encoding if defined $encoding;
+    $parser->{HANDLER}->xml_decl($decl);
+}
+
+sub start_element {
+    my (  $parser, $elem, $attrs ) = @_;
+    my $saxattr = {};
+
+    push @{$parser->{SAX}->{ELSTACK}}, $elem;
+    if ( defined $attrs ) {
+        $parser->{HANDLER}->start_element( { %$elem, Attributes=>$attrs} )
+    }
+
+}
+
+sub end_element {
+    my (  $parser, $name ) = @_;
+    my $elem = pop @{$parser->{SAX}->{ELSTACK}};
+    if ( $elem->{Name} ne $name ) {
+        croak( "cought error where parser should work ($elem->{Name} != $name" );
+    }
+    $parser->{HANDLER}->end_element( $elem );
+}
+
+sub characters {
+    my ( $parser, $data ) = @_;
+    $parser->{HANDLER}->characters( {Data => $data} );
+}
+
+sub comment {
+    my ( $parser, $data ) = @_;
+    $parser->{HANDLER}->comment( {Data => $data} );
+}
+
+sub cdata_block {
+    my ( $parser, $data ) = @_;
+    $parser->{HANDLER}->start_cdata();
+    $parser->{HANDLER}->characters( {Data => $data} );
+    $parser->{HANDLER}->end_cdata();
+}
+
+sub processing_instruction {
+    my ( $parser, $target, $data ) = @_;
+    $parser->{HANDLER}->processing_instruction( {Target => $target,
+                                                 Data   => $data} );
+}
+
+1;
 __END__
 
 =head1 NAME
@@ -961,6 +1164,60 @@ should to be used for the XInclude as well.
 If expand_xincludes is set to 1, the method is only required to process
 XIncludes appended to the DOM after its original parsing.
 
+=head1 PUSH PARSER
+
+XML::LibXML supports also a push parser interface. This allows one to
+parse large documents without actually loading the entire document
+into memory.
+
+The interface is devided into two parts:
+
+=over 4
+
+=item * pushing the data into the parser
+
+=item * finish the parse
+
+=back
+
+The user has no chance to access the document while still pushing the
+data to the parser. The resulting document will be returned when the
+parser is told to finish the parsing process.
+
+=over 4
+
+=item $parser->push( @data )
+
+This function pushs the data stored inside the array to libxml2's
+parse. Each entry in @data must be a normal scalar!
+
+=item $parser->finish( $restore );
+
+This function returns the result of the parsing process. If this
+function is called without a parameter it will complain about non
+wellformed documents. If $restore is 1, the push parser can be used to
+restore broken or non well formed (XML) documents as the following
+example shows:
+
+  $parser->push( "<foo>", "bar" );
+  eval { $doc = $parser->finish; };      # will complain
+  if ( $@ ) {
+     # ...
+  }
+
+This can be anoing if the closing tag misses by accident. The
+following code will restore the document:
+
+  $parser->push( "<foo>", "bar" );
+  eval { $doc = $parser->finish(1); };      # will not complain
+
+  warn $doc->toString(); # returns "<foo>bar</foo>"
+
+of course finish() will return nothing if there was no data pushed to
+the parser before.
+
+=back
+
 =head1 SERIALIZATION
 
 The oposite of parsing is serialization. In XML::LibXML this can be
@@ -1021,20 +1278,6 @@ L<"XML::LibXML::Dtd">.
 Process any xinclude tags in the file. (currently using B<only> libxml2's
 default callbacks)
 
-
-=head1 XML::LibXML::Dtd
-
-This module allows you to parse and return a DTD object. It has one method
-right now, C<new()>.
-
-=head2 new()
-
-  my $dtd = XML::LibXML::Dtd->new($public, $system);
-
-Creates a new DTD object from the public and system identifiers. It will
-automatically load the objects from the filesystem, or use the input
-callbacks (see L<"Input Callbacks"> below) to load the DTD.
-
 =head1 Input Callbacks
 
 The input callbacks are used whenever LibXML has to get something B<other
@@ -1052,23 +1295,27 @@ opened streams.
 
 The following callbacks are defined:
 
-=head2 match(uri)
+=over 4
+
+=item match(uri)
 
 If you want to handle the URI, simply return a true value from this callback.
 
-=head2 open(uri)
+=item open(uri)
 
 Open something and return it to handle that resource.
 
-=head2 read(handle, bytes)
+=item read(handle, bytes)
 
 Read a certain number of bytes from the resource. This callback is
 called even if the entire Document has already read.
 
-=head2 close(handle)
+=item close(handle)
 
 Close the handle associated with the resource.
 
+=back
+
 =head2 Example
 
 This is a purely fictitious example that uses a MyScheme::Handler object
@@ -1107,6 +1354,27 @@ that responds to methods similar to an IO::Handle.
 
 A more realistic example can be found in the L<"example"> directory
 
+Since the parser requires all callbacks defined it is also possible to
+set all callbacks with a single call of callbacks(). This would
+simplify the example code to:
+
+  $parser->callbacks( \&match_uri, \&open_uri, \&read_uri, \&close_uri);
+
+All functions that are used to set the callbacks, can also be used to
+retrieve the callbacks from the parser.
+
+=head2 Global Callbacks
+
+Optionaly it is possible to apply global callback on the XML::LibXML
+class level. This allows multiple parses to share the same callbacks.
+To set these global callbacks one can use the callback access
+functions directly on the class.
+
+  XML::LibXML->callbacks( \&match_uri, \&open_uri, \&read_uri, \&close_uri);
+
+The previous code snippet will set the callbacks from the first
+example as global callbacks.
+
 =head1 Encoding
 
 All data will be stored UTF-8 encoded. Nevertheless the input and
@@ -1147,6 +1415,19 @@ encoding.  While transforms to ISO encodings may cause errors if the
 given stirng contains unsupported characters, this function can
 transform to UTF-16 encodings as well.
 
+=head1 XML::LibXML::Dtd
+
+This module allows you to parse and return a DTD object. It has one method
+right now, C<new()>.
+
+=head2 new()
+
+  my $dtd = XML::LibXML::Dtd->new($public, $system);
+
+Creates a new DTD object from the public and system identifiers. It will
+automatically load the objects from the filesystem, or use the input
+callbacks (see L<"Input Callbacks"> below) to load the DTD.
+
 =head1 Processing Instructions - XML::LibXML::PI
 
 Processing instructions are implemented with XML::LibXML with read and
@@ -1235,10 +1516,6 @@ Copyright 2001, AxKit.com Ltd. All rights reserved.
 
 =head1 SEE ALSO
 
-L<XML::LibXSLT>, L<XML::LibXML::DOM>, L<XML::LibXML::Document>,
-L<XML::LibXML::Element>, L<XML::LibXML::Node>,
-L<XML::LibXML::Text>, L<XML::LibXML::Comment>,
-L<XML::LibXML::CDATASection>, L<XML::LibXML::Attribute>
-L<XML::LibXML::DocumentFragment>
+L<XML::LibXSLT>, L<XML::LibXML::DOM>, L<XML::LibXML::SAX>
 
 =cut
