@@ -1,4 +1,4 @@
-# $Id: Builder.pm,v 1.6 2002/05/08 00:27:06 phish Exp $
+# $Id: Builder.pm,v 1.11 2002/05/20 11:01:28 phish Exp $
 
 package XML::LibXML::SAX::Builder;
 
@@ -35,8 +35,9 @@ sub xml_decl {
 
 sub end_document {
     my ($self, $doc) = @_;
-    my $dom = delete $self->{DOM};
+    my $dom = $self->{DOM};
     delete $self->{Parent};
+    delete $self->{DOM};
     return $dom;
 }
 
@@ -50,18 +51,6 @@ sub start_element {
         $node = $self->{DOM}->createElement($el->{Name});
     }
 
-    # do attributes
-    foreach my $key (keys %{$el->{Attributes}}) {
-        my $attr = $el->{Attributes}->{$key};
-        if (ref($attr)) {
-            # SAX2 attributes
-            $node->setAttributeNS($attr->{NamespaceURI} || "", $attr->{Name} => $attr->{Value});
-        }
-        else {
-            $node->setAttribute($key => $attr);
-        }
-    }
-
     # append
     if ($self->{Parent}) {
         $self->{Parent}->appendChild($node);
@@ -71,6 +60,24 @@ sub start_element {
         $self->{DOM}->setDocumentElement($node);
         $self->{Parent} = $node;
     }
+
+    # do attributes
+    foreach my $key (keys %{$el->{Attributes}}) {
+        my $attr = $el->{Attributes}->{$key};
+        if (ref($attr)) {
+            if ( not defined $attr->{Prefix} or $attr->{Prefix} ne "xmlns" ) {
+                # SAX2 attributes
+                $node->setAttributeNS($attr->{NamespaceURI} || "",
+                                      $attr->{Name}, $attr->{Value});
+            }
+            else {
+                $node->setNamespace( $attr->{Value}, $attr->{LocalName},0 );
+            }
+        }
+        else {
+            $node->setAttribute($key => $attr);
+        }
+    }
 }
 
 sub end_element {
@@ -85,6 +92,28 @@ sub characters {
     $self->{Parent}->appendText($chars->{Data});
 }
 
+sub comment {
+    my ($self, $chars) = @_;
+    my $comment = $self->{DOM}->createComment( $chars->{Data} );
+    if ( defined $self->{Parent} ) {
+        $self->{Parent}->appendChild($comment);
+    }
+    else {
+        $self->{DOM}->appendChild($comment);
+    }
+}
+
+sub processing_instruction {
+    my ( $self,  $pi ) = @_;
+    my $PI = $self->{DOM}->createPI( $pi->{Target}, $pi->{Data} );
+    if ( defined $self->{Parent} ) {
+        $self->{Parent}->appendChild( $PI );
+    }
+    else {
+        $self->{DOM}->appendChild( $PI );
+    }
+}
+
 1;
 
 __END__
